#!/bin/bash

set -e

getInstances() {
    pgrep -f "redis-server"
}

getRedisCommand() {
    PID=$1
    INFO=$(lsof -P -a -p "${PID}" 2>/dev/null | while read -r LINE; do
        FD=$(echo "${LINE}" | awk '{ print $4 }')
        TYPE=$(echo "${LINE}" | awk '{ print $5 }')
        VALUE=$(echo "${LINE}" | awk '{ print $9 }')
        if [ -z "${VALUE}" ]; then
            VALUE=$(echo "${LINE}" | awk '{ print $8 }')
        fi
        LISTEN=$(echo "${LINE}" | grep '(LISTEN)' > /dev/null; echo $?)

        if [ "${FD}" = 'cwd' ] && [ "${TYPE}" = 'DIR' ]; then
            echo "WORKING_DIR ${VALUE}";
        elif [ "${TYPE}" = 'unix' ]; then
            echo "SOCKET ${VALUE}";
        elif [ "${TYPE}" = 'IPv4' ] && [ "${LISTEN}" = 0 ]; then
            VALUE=$(echo "${VALUE}" | sed 's/^\([0-9\.]\{1,\}\|*\)://')
            if [[ "${VALUE}" == *":"* ]]; then
                VALUE=$(echo "${VALUE}" | cut -d: -f2)
            fi
            echo "PORT ${VALUE}";
        fi
    done);

    WORKING_DIR=$(echo "${INFO}" | grep 'WORKING_DIR' | awk '{ print $2 }')
    SOCKET=$(echo "${INFO}" | grep 'SOCKET' | awk '{ print $2 }')
    PORT=$(echo "${INFO}" | grep 'PORT' | head -n1 | awk '{ print $2 }')
    if [ -n "${PORT}" ]; then
        redis-cli -p "${PORT}" info | parseRedisInfo "${PORT}"
    else
        if [[ ! "${SOCKET}" = /* ]]; then
            SOCKET="${WORKING_DIR}/${SOCKET}"
        fi

        redis-cli -s "${SOCKET}" info | parseRedisInfo "${SOCKET}"
    fi
}

parseRedisInfo() {
    TARGET=$1
    while read -r LINE; do
        if [[ $LINE =~ ^(used_memory|maxmemory|total_|instantaneous_|sync_|expired_|evicted_|keyspace_|active_defrag_|connected_slaves|master_repl_|repl_backlog_|connected_clients|maxclients|blocked_clients|tracking_clients|clients_in_timeout_table|db[0-9]+) ]]; then
            STAT=$(echo "${LINE}" | awk -F: '{ print $1 }')
            VALUE=$(echo "${LINE}" | awk -F: '{ print $2 }' | sed 's/\r//g')

            if [[ $STAT =~ ^db[0-9]+$ ]]; then
                # keys=258,expires=0,avg_ttl=0
                for DB_PART in $(echo "${VALUE}" | sed "s/,/ /g"); do
                    DB_STAT=$(echo "${DB_PART}" | awk -F= '{ print $1 }')
                    DB_VALUE=$(echo "${DB_PART}" | awk -F= '{ print $2 }')
                    echo "redis_db_${DB_STAT}{target=\"${TARGET}\",db=\"${STAT}\"} ${DB_VALUE}"
                done
            elif ! [[ $STAT =~ (_human|_perc|_policy)$ ]]; then
                echo "redis_${STAT}{target=\"${TARGET}\"} ${VALUE}"
            fi
        fi
    done
}

for PID in $(getInstances); do
    getRedisCommand "${PID}"
done
